@@ -97,6 +97,7 @@ import org.apache.fory.codegen.Expression.ListExpression;
 import org.apache.fory.codegen.Expression.Literal;
 import org.apache.fory.codegen.Expression.Reference;
 import org.apache.fory.codegen.Expression.Return;
+import org.apache.fory.codegen.Expression.Variable;
 import org.apache.fory.codegen.Expression.While;
 import org.apache.fory.codegen.ExpressionUtils;
 import org.apache.fory.codegen.ExpressionVisitor.ExprHolder;
@@ -131,11 +132,11 @@ import org.apache.fory.util.StringUtils;
  */
 @SuppressWarnings("unchecked")
 public abstract class BaseObjectCodecBuilder extends CodecBuilder {
-  public static final String BUFFER_NAME = "buffer";
-  public static final String REF_RESOLVER_NAME = "refResolver";
-  public static final String CLASS_RESOLVER_NAME = "classResolver";
-  public static final String POJO_CLASS_TYPE_NAME = "classType";
-  public static final String STRING_SERIALIZER_NAME = "strSerializer";
+  public static final String BUFFER_NAME = "_f_buffer";
+  public static final String REF_RESOLVER_NAME = "_f_refResolver";
+  public static final String CLASS_RESOLVER_NAME = "_f_classResolver";
+  public static final String POJO_CLASS_TYPE_NAME = "_f_classType";
+  public static final String STRING_SERIALIZER_NAME = "_f_strSerializer";
   private static final TypeRef<?> CLASS_RESOLVER_TYPE_TOKEN = TypeRef.of(ClassResolver.class);
   private static final TypeRef<?> STRING_SERIALIZER_TYPE_TOKEN = TypeRef.of(StringSerializer.class);
   private static final TypeRef<?> SERIALIZER_TYPE = TypeRef.of(Serializer.class);
@@ -266,7 +267,7 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
         ROOT_OBJECT_NAME);
     ctx.overrideMethod("read", decodeCode, Object.class, MemoryBuffer.class, BUFFER_NAME);
     registerJITNotifyCallback();
-    ctx.addConstructor(constructorCode, Fory.class, "fory", Class.class, POJO_CLASS_TYPE_NAME);
+    ctx.addConstructor(constructorCode, Fory.class, FORY_NAME, Class.class, POJO_CLASS_TYPE_NAME);
     return ctx.genCode();
   }
 
@@ -948,7 +949,10 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
       builder.add(action);
     }
     walkPath.removeLast();
-    return new ListExpression(onCollectionWrite, new If(gt(size, ofInt(0)), builder));
+    return new ListExpression(
+        onCollectionWrite,
+        new If(gt(size, ofInt(0)), builder),
+        new Invoke(serializer, "onCollectionWriteFinish", collection));
   }
 
   /**
@@ -1237,8 +1241,9 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
                   new If(
                       neqNull(entry), inline ? writeChunk : new Assign(entry, inline(writeChunk))));
             });
-
-    return new If(not(inlineInvoke(map, "isEmpty", PRIMITIVE_BOOLEAN_TYPE)), whileAction);
+    return new ListExpression(
+        new If(not(inlineInvoke(map, "isEmpty", PRIMITIVE_BOOLEAN_TYPE)), whileAction),
+        new Invoke(serializer, "onMapWriteFinish", map));
   }
 
   private Tuple2<Expression, Expression> getMapKVSerializer(Class<?> keyType, Class<?> valueType) {
@@ -1268,12 +1273,16 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
       TypeRef<?> keyType,
       TypeRef<?> valueType) {
     ListExpression expressions = new ListExpression();
-    Expression key = invoke(entry, "getKey", "key", keyType);
-    Expression value = invoke(entry, "getValue", "value", valueType);
     boolean keyMonomorphic = isMonomorphic(keyType);
     boolean valueMonomorphic = isMonomorphic(valueType);
     Class<?> keyTypeRawType = keyType.getRawType();
     Class<?> valueTypeRawType = valueType.getRawType();
+    Expression key =
+        keyMonomorphic ? new Variable("key", keyType) : invoke(entry, "getKey", "key", keyType);
+    Expression value =
+        valueMonomorphic
+            ? new Variable("value", valueType)
+            : invoke(entry, "getValue", "value", valueType);
     Expression keyTypeExpr =
         keyMonomorphic
             ? getClassExpr(keyTypeRawType)
@@ -1395,6 +1404,9 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
         new While(
             Literal.ofBoolean(true),
             () -> {
+              Expression keyAssign = new Assign(key, invokeInline(entry, "getKey", keyType));
+              Expression valueAssign =
+                  new Assign(value, invokeInline(entry, "getValue", valueType));
               Expression breakCondition;
               if (keyMonomorphic && valueMonomorphic) {
                 breakCondition = or(eqNull(key), eqNull(value));
@@ -1450,20 +1462,16 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
                         writeValue);
               }
               return new ListExpression(
+                  keyAssign,
+                  valueAssign,
                   new If(breakCondition, new Break()),
                   writeKey,
                   writeValue,
                   new Assign(chunkSize, add(chunkSize, ofInt(1))),
                   new If(
                       inlineInvoke(iterator, "hasNext", PRIMITIVE_BOOLEAN_TYPE),
-                      new ListExpression(
-                          new Assign(
-                              entry,
-                              cast(inlineInvoke(iterator, "next", OBJECT_TYPE), MAP_ENTRY_TYPE)),
-                          new Assign(
-                              key,
-                              tryInlineCast(inlineInvoke(entry, "getKey", OBJECT_TYPE), keyType)),
-                          new Assign(value, invokeInline(entry, "getValue", valueType))),
+                      new Assign(
+                          entry, cast(inlineInvoke(iterator, "next", OBJECT_TYPE), MAP_ENTRY_TYPE)),
                       list(new Assign(entry, new Literal(null, MAP_ENTRY_TYPE)), new Break())),
                   new If(eq(chunkSize, ofInt(MAX_CHUNK_SIZE)), new Break()));
             });
@@ -1636,13 +1644,13 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
         obj = deserializeForMap(buffer, typeRef, serializer, invokeHint);
       } else {
         if (serializer != null) {
-          return new Invoke(serializer, "read", OBJECT_TYPE, buffer);
+          return read(serializer, buffer, OBJECT_TYPE);
         }
         if (isMonomorphic(cls)) {
           serializer = getOrCreateSerializer(cls);
           Class<?> returnType =
               ReflectionUtils.getReturnType(getRawType(serializer.type()), "read");
-          obj = new Invoke(serializer, "read", TypeRef.of(returnType), buffer);
+          obj = read(serializer, buffer, TypeRef.of(returnType));
         } else {
           obj = readForNotNullNonFinal(buffer, typeRef, serializer);
         }
@@ -1651,13 +1659,24 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
     }
   }
 
+  protected Expression read(Expression serializer, Expression buffer, TypeRef<?> returnType) {
+    Class<?> type = returnType.getRawType();
+    Expression read = new Invoke(serializer, "read", returnType, buffer);
+    if (ReflectionUtils.isMonomorphic(type) && !TypeUtils.hasExpandableLeafs(type)) {
+      return read;
+    }
+    read = uninline(read);
+    return new ListExpression(
+        new Invoke(foryRef, "incReadDepth"), read, new Invoke(foryRef, "decDepth"), read);
+  }
+
   protected Expression readForNotNullNonFinal(
       Expression buffer, TypeRef<?> typeRef, Expression serializer) {
     if (serializer == null) {
       Expression classInfo = readClassInfo(getRawType(typeRef), buffer);
       serializer = inlineInvoke(classInfo, "getSerializer", SERIALIZER_TYPE);
     }
-    return new Invoke(serializer, "read", OBJECT_TYPE, buffer);
+    return read(serializer, buffer, OBJECT_TYPE);
   }
 
   /**
@@ -1693,7 +1712,7 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
         new If(
             supportHook,
             new ListExpression(collection, hookRead),
-            new Invoke(serializer, "read", OBJECT_TYPE, buffer),
+            read(serializer, buffer, OBJECT_TYPE),
             false);
     if (invokeHint != null && invokeHint.genNewMethod) {
       invokeHint.add(buffer);
@@ -1969,8 +1988,7 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
     expressions.add(chunksLoop, newMap);
     // first newMap to create map, last newMap as expr value
     Expression map = inlineInvoke(serializer, "onMapRead", OBJECT_TYPE, expressions);
-    Expression action =
-        new If(supportHook, map, new Invoke(serializer, "read", OBJECT_TYPE, buffer), false);
+    Expression action = new If(supportHook, map, read(serializer, buffer, OBJECT_TYPE), false);
     if (invokeHint != null && invokeHint.genNewMethod) {
       invokeHint.add(buffer);
       invokeHint.add(serializer);
