@@ -37,6 +37,7 @@ import com.google.common.collect.HashBiMap;
 import java.io.Externalizable;
 import java.io.IOException;
 import java.io.Serializable;
+import java.lang.invoke.SerializedLambda;
 import java.lang.reflect.Field;
 import java.lang.reflect.Member;
 import java.lang.reflect.Type;
@@ -87,7 +88,8 @@ import org.apache.fory.ForyCopyable;
 import org.apache.fory.annotation.CodegenInvoke;
 import org.apache.fory.annotation.Internal;
 import org.apache.fory.builder.CodecUtils;
-import org.apache.fory.builder.Generated;
+import org.apache.fory.builder.Generated.GeneratedMetaSharedSerializer;
+import org.apache.fory.builder.Generated.GeneratedObjectSerializer;
 import org.apache.fory.builder.JITContext;
 import org.apache.fory.codegen.CodeGenerator;
 import org.apache.fory.codegen.Expression;
@@ -163,6 +165,7 @@ import org.apache.fory.type.ScalaTypes;
 import org.apache.fory.type.TypeUtils;
 import org.apache.fory.type.Types;
 import org.apache.fory.util.GraalvmSupport;
+import org.apache.fory.util.GraalvmSupport.GraalvmSerializerHolder;
 import org.apache.fory.util.Preconditions;
 import org.apache.fory.util.StringUtils;
 import org.apache.fory.util.function.Functions;
@@ -420,6 +423,7 @@ public class ClassResolver implements TypeResolver {
     register(AtomicReference.class);
     register(EnumSet.allOf(Language.class).getClass());
     register(EnumSet.of(Language.JAVA).getClass());
+    register(SerializedLambda.class);
     register(Throwable.class, StackTraceElement.class, Exception.class, RuntimeException.class);
     register(NullPointerException.class);
     register(IOException.class);
@@ -968,14 +972,14 @@ public class ClassResolver implements TypeResolver {
         return CollectionSerializers.EnumSetSerializer.class;
       } else if (Charset.class.isAssignableFrom(cls)) {
         return Serializers.CharsetSerializer.class;
-      } else if (Functions.isLambda(cls)) {
-        return LambdaSerializer.class;
       } else if (ReflectionUtils.isJdkProxy(cls)) {
         if (JavaSerializer.getWriteReplaceMethod(cls) != null) {
           return ReplaceResolveSerializer.class;
         } else {
           return JdkProxySerializer.class;
         }
+      } else if (Functions.isLambda(cls)) {
+        return LambdaSerializer.class;
       } else if (Calendar.class.isAssignableFrom(cls)) {
         return TimeSerializers.CalendarSerializer.class;
       } else if (ZoneId.class.isAssignableFrom(cls)) {
@@ -1423,17 +1427,21 @@ public class ClassResolver implements TypeResolver {
       if (deserializationClassInfo != null && GraalvmSupport.isGraalBuildtime()) {
         getGraalvmClassRegistry()
             .deserializerClassMap
-            .put(classDef.getId(), deserializationClassInfo.serializer.getClass());
+            .put(classDef.getId(), getGraalvmSerializerClass(deserializationClassInfo.serializer));
         Tuple2<ClassDef, ClassInfo> classDefTuple = extRegistry.classIdToDef.get(classDef.getId());
         // empty serializer for graalvm build time
         classDefTuple.f1.serializer = null;
+        classInfoCache = NIL_CLASS_INFO;
         extRegistry.classIdToDef.put(classDef.getId(), Tuple2.of(classDefTuple.f0, null));
       }
     }
     if (GraalvmSupport.isGraalBuildtime()) {
       // Instance for generated class should be hold at graalvm runtime only.
-      getGraalvmClassRegistry().serializerClassMap.put(cls, classInfo.serializer.getClass());
+      getGraalvmClassRegistry()
+          .serializerClassMap
+          .put(cls, getGraalvmSerializerClass(classInfo.serializer));
       classInfo.serializer = null;
+      classInfoCache = NIL_CLASS_INFO;
     }
   }
 
@@ -1560,13 +1568,21 @@ public class ClassResolver implements TypeResolver {
   }
 
   boolean needToWriteClassDef(Serializer serializer) {
-    return fory.getConfig().getCompatibleMode() == CompatibleMode.COMPATIBLE
-        && (serializer instanceof Generated.GeneratedObjectSerializer
-            // May already switched to MetaSharedSerializer when update class info cache.
-            || serializer instanceof Generated.GeneratedMetaSharedSerializer
-            || serializer instanceof LazyInitBeanSerializer
-            || serializer instanceof ObjectSerializer
-            || serializer instanceof MetaSharedSerializer);
+    if (fory.getConfig().getCompatibleMode() != CompatibleMode.COMPATIBLE) {
+      return false;
+    }
+    if (GraalvmSupport.isGraalBuildtime() && serializer instanceof GraalvmSerializerHolder) {
+      Class<? extends Serializer> serializerClass =
+          ((GraalvmSerializerHolder) serializer).getSerializerClass();
+      return GeneratedObjectSerializer.class.isAssignableFrom(serializerClass)
+          || GeneratedMetaSharedSerializer.class.isAssignableFrom(serializerClass);
+    }
+    return (serializer instanceof GeneratedObjectSerializer
+        // May already switched to MetaSharedSerializer when update class info cache.
+        || serializer instanceof GeneratedMetaSharedSerializer
+        || serializer instanceof LazyInitBeanSerializer
+        || serializer instanceof ObjectSerializer
+        || serializer instanceof MetaSharedSerializer);
   }
 
   private ClassInfo readClassInfoWithMetaShare(MemoryBuffer buffer, MetaContext metaContext) {
@@ -2048,20 +2064,63 @@ public class ClassResolver implements TypeResolver {
 
   public void resetWrite() {}
 
+  private static final GenericType OBJECT_GENERIC_TYPE = GenericType.build(Object.class);
+
   @CodegenInvoke
   public GenericType getGenericTypeInStruct(Class<?> cls, String genericTypeStr) {
     Map<String, GenericType> map =
-        extRegistry.classGenericTypes.computeIfAbsent(cls, k -> new HashMap<>());
-    GenericType genericType = map.get(genericTypeStr);
-    if (genericType == null) {
-      for (Field field : ReflectionUtils.getFields(cls, true)) {
-        Type type = field.getGenericType();
-        TypeRef<Object> typeRef = TypeRef.of(type);
-        genericType = buildGenericType(typeRef);
-        map.put(type.getTypeName(), genericType);
-      }
+        extRegistry.classGenericTypes.computeIfAbsent(cls, this::buildGenericMap);
+    return map.getOrDefault(genericTypeStr, OBJECT_GENERIC_TYPE);
+  }
+
+  /**
+   * Build a map of nested generic type name to generic type for all fields in the class.
+   *
+   * @param cls the class to build the map of nested generic type name to generic type for all
+   *     fields in the class
+   * @return a map of nested generic type name to generic type for all fields in the class
+   */
+  protected Map<String, GenericType> buildGenericMap(Class<?> cls) {
+    Map<String, GenericType> map = new HashMap<>();
+    Map<String, GenericType> map2 = new HashMap<>();
+    for (Field field : ReflectionUtils.getFields(cls, true)) {
+      Type type = field.getGenericType();
+      GenericType genericType = buildGenericType(type);
+      buildGenericMap(map, genericType);
+      TypeRef<?> typeRef = TypeRef.of(type);
+      buildGenericMap(map2, typeRef);
+    }
+    map.putAll(map2);
+    return map;
+  }
+
+  private void buildGenericMap(Map<String, GenericType> map, TypeRef<?> typeRef) {
+    if (map.containsKey(typeRef.getType().getTypeName())) {
+      return;
+    }
+    map.put(typeRef.getType().getTypeName(), buildGenericType(typeRef));
+    Class<?> rawType = typeRef.getRawType();
+    if (TypeUtils.isMap(rawType)) {
+      Tuple2<TypeRef<?>, TypeRef<?>> kvTypes = TypeUtils.getMapKeyValueType(typeRef);
+      buildGenericMap(map, kvTypes.f0);
+      buildGenericMap(map, kvTypes.f1);
+    } else if (TypeUtils.isCollection(rawType)) {
+      TypeRef<?> elementType = TypeUtils.getElementType(typeRef);
+      buildGenericMap(map, elementType);
+    } else if (rawType.isArray()) {
+      TypeRef<?> arrayComponent = TypeUtils.getArrayComponent(typeRef);
+      buildGenericMap(map, arrayComponent);
+    }
+  }
+
+  private void buildGenericMap(Map<String, GenericType> map, GenericType genericType) {
+    if (map.containsKey(genericType.getType().getTypeName())) {
+      return;
+    }
+    map.put(genericType.getType().getTypeName(), genericType);
+    for (GenericType t : genericType.getTypeParameters()) {
+      buildGenericMap(map, t);
     }
-    return genericType;
   }
 
   @Override
@@ -2212,6 +2271,10 @@ public class ClassResolver implements TypeResolver {
    */
   public void ensureSerializersCompiled() {
     try {
+      fory.getJITContext().lock();
+      Serializers.newSerializer(fory, LambdaSerializer.STUB_LAMBDA_CLASS, LambdaSerializer.class);
+      Serializers.newSerializer(
+          fory, JdkProxySerializer.SUBT_PROXY.getClass(), JdkProxySerializer.class);
       classInfoMap.forEach(
           (cls, classInfo) -> {
             if (classInfo.serializer == null) {
@@ -2227,6 +2290,7 @@ public class ClassResolver implements TypeResolver {
               }
             });
       }
+      classInfoCache = NIL_CLASS_INFO;
     } finally {
       fory.getJITContext().unlock();
     }
@@ -2262,6 +2326,13 @@ public class ClassResolver implements TypeResolver {
         fory.getConfig().getConfigHash(), k -> new GraalvmClassRegistry());
   }
 
+  private Class<? extends Serializer> getGraalvmSerializerClass(Serializer serializer) {
+    if (serializer instanceof GraalvmSerializerHolder) {
+      return ((GraalvmSerializerHolder) serializer).getSerializerClass();
+    }
+    return serializer.getClass();
+  }
+
   private Class<? extends Serializer> getSerializerClassFromGraalvmRegistry(Class<?> cls) {
     GraalvmClassRegistry registry = getGraalvmClassRegistry();
     List<ClassResolver> classResolvers = registry.resolvers;
@@ -2307,7 +2378,10 @@ public class ClassResolver implements TypeResolver {
       if (Functions.isLambda(cls) || ReflectionUtils.isJdkProxy(cls)) {
         return null;
       }
-      throw new RuntimeException(String.format("Class %s is not registered", cls));
+      throw new RuntimeException(
+          String.format(
+              "Class %s is not registered, registered classes: %s",
+              cls, registry.deserializerClassMap));
     }
     return null;
   }
