@@ -116,10 +116,13 @@ public abstract class CollectionLikeSerializer<T> extends Serializer<T> {
    *   <li>write collection size
    *   <li>onCollectionWrite
    *   <li>write elements
+   *   <li>onCollectionWriteFinish
    * </ol>
    */
   public abstract Collection onCollectionWrite(MemoryBuffer buffer, T value);
 
+  public void onCollectionWriteFinish(Collection map) {}
+
   /**
    * Write elements data header. Keep this consistent with
    * `BaseObjectCodecBuilder#writeElementsHeader`.
@@ -313,6 +316,7 @@ public abstract class CollectionLikeSerializer<T> extends Serializer<T> {
     if (len != 0) {
       writeElements(fory, buffer, collection);
     }
+    onCollectionWriteFinish(collection);
   }
 
   protected final void writeElements(Fory fory, MemoryBuffer buffer, Collection value) {
@@ -662,7 +666,7 @@ public abstract class CollectionLikeSerializer<T> extends Serializer<T> {
       int flags,
       T collection,
       int numElements) {
-    fory.incDepth(1);
+    fory.incReadDepth();
     if ((flags & CollectionFlags.TRACKING_REF) == CollectionFlags.TRACKING_REF) {
       for (int i = 0; i < numElements; i++) {
         collection.add(binding.readRef(buffer, serializer));
@@ -682,7 +686,7 @@ public abstract class CollectionLikeSerializer<T> extends Serializer<T> {
         }
       }
     }
-    fory.incDepth(-1);
+    fory.decDepth();
   }
 
   /** Read elements whose type are different. */
