@@ -168,6 +168,7 @@ public abstract class MapLikeSerializer<T> extends Serializer<T> {
         }
       }
     }
+    onMapWriteFinish(map);
   }
 
   @Override
@@ -689,11 +690,13 @@ public abstract class MapLikeSerializer<T> extends Serializer<T> {
             if (keySerializer == null) {
               key = readNonEmptyValueFromNullChunk(buffer, trackKeyRef, true);
             } else {
+              fory.incReadDepth();
               if (trackKeyRef) {
                 key = binding.readRef(buffer, keySerializer);
               } else {
                 key = binding.read(buffer, keySerializer);
               }
+              fory.decDepth();
             }
           } else {
             key = binding.readRef(buffer, keyClassInfoReadCache);
@@ -729,11 +732,13 @@ public abstract class MapLikeSerializer<T> extends Serializer<T> {
         if (valueSerializer == null) {
           value = readNonEmptyValueFromNullChunk(buffer, trackValueRef, false);
         } else {
+          fory.incReadDepth();
           if (trackValueRef) {
             value = binding.readRef(buffer, valueSerializer);
           } else {
             value = binding.read(buffer, valueSerializer);
           }
+          fory.decDepth();
         }
       } else {
         value = binding.readRef(buffer, valueClassInfoReadCache);
@@ -753,15 +758,15 @@ public abstract class MapLikeSerializer<T> extends Serializer<T> {
     }
     GenericType type = isKey ? genericType.getTypeParameter0() : genericType.getTypeParameter1();
     generics.pushGenericType(type);
-    fory.incDepth(1);
     Serializer<?> serializer = type.getSerializer(typeResolver);
     Object v;
+    fory.incReadDepth();
     if (trackRef) {
       v = binding.readRef(buffer, serializer);
     } else {
       v = binding.read(buffer, serializer);
     }
-    fory.incDepth(-1);
+    fory.decDepth();
     generics.popGenericType();
     return v;
   }
@@ -781,8 +786,10 @@ public abstract class MapLikeSerializer<T> extends Serializer<T> {
         if (!valueHasNull) {
           return (size << 8) | chunkHeader;
         } else {
+          fory.incReadDepth();
           Object key = binding.read(buffer, keySerializer);
           map.put(key, null);
+          fory.decDepth();
         }
       } else {
         readNullKeyChunk(buffer, map, chunkHeader, valueSerializer, valueHasNull);
@@ -815,6 +822,7 @@ public abstract class MapLikeSerializer<T> extends Serializer<T> {
     if (!valueIsDeclaredType) {
       valueSerializer = typeResolver.readClassInfo(buffer, valueClassInfoReadCache).getSerializer();
     }
+    fory.incReadDepth();
     for (int i = 0; i < chunkSize; i++) {
       Object key =
           trackKeyRef
@@ -827,6 +835,7 @@ public abstract class MapLikeSerializer<T> extends Serializer<T> {
       map.put(key, value);
       size--;
     }
+    fory.decDepth();
     return size > 0 ? (size << 8) | buffer.readUnsignedByte() : 0;
   }
 
@@ -866,20 +875,20 @@ public abstract class MapLikeSerializer<T> extends Serializer<T> {
     if (keyGenericType.hasGenericParameters() || valueGenericType.hasGenericParameters()) {
       for (int i = 0; i < chunkSize; i++) {
         generics.pushGenericType(keyGenericType);
-        fory.incDepth(1);
+        fory.incReadDepth();
         Object key =
             trackKeyRef
                 ? binding.readRef(buffer, keySerializer)
                 : binding.read(buffer, keySerializer);
-        fory.incDepth(-1);
+        fory.decDepth();
         generics.popGenericType();
         generics.pushGenericType(valueGenericType);
-        fory.incDepth(1);
+        fory.incReadDepth();
         Object value =
             trackValueRef
                 ? binding.readRef(buffer, valueSerializer)
                 : binding.read(buffer, valueSerializer);
-        fory.incDepth(-1);
+        fory.decDepth();
         generics.popGenericType();
         map.put(key, value);
         size--;
@@ -887,7 +896,7 @@ public abstract class MapLikeSerializer<T> extends Serializer<T> {
     } else {
       for (int i = 0; i < chunkSize; i++) {
         // increase depth to avoid read wrong outer generic type
-        fory.incDepth(1);
+        fory.incReadDepth();
         Object key =
             trackKeyRef
                 ? binding.readRef(buffer, keySerializer)
@@ -896,7 +905,7 @@ public abstract class MapLikeSerializer<T> extends Serializer<T> {
             trackValueRef
                 ? binding.readRef(buffer, valueSerializer)
                 : binding.read(buffer, valueSerializer);
-        fory.incDepth(-1);
+        fory.decDepth();
         map.put(key, value);
         size--;
       }
@@ -928,6 +937,8 @@ public abstract class MapLikeSerializer<T> extends Serializer<T> {
    */
   public abstract Map onMapWrite(MemoryBuffer buffer, T value);
 
+  public void onMapWriteFinish(Map map) {}
+
   /**
    * Read data except size and elements, return empty map to be filled.
    *
