@@ -18,7 +18,6 @@
 import array
 import datetime
 import gc
-import io
 import os
 import pickle
 import weakref
@@ -308,37 +307,6 @@ def ser_de(fory, obj):
     return fory.deserialize(binary)
 
 
-def test_pickle():
-    buf = Buffer.allocate(32)
-    pickler = pickle.Pickler(buf)
-    pickler.dump(b"abc")
-    buf.write_int32(-1)
-    pickler.dump("abcd")
-    assert buf.writer_index - 4 == len(pickle.dumps(b"abc")) + len(pickle.dumps("abcd"))
-    print(f"writer_index {buf.writer_index}")
-
-    bytes_io_ = io.BytesIO(buf)
-    unpickler = pickle.Unpickler(bytes_io_)
-    assert unpickler.load() == b"abc"
-    bytes_io_.seek(bytes_io_.tell() + 4)
-    assert unpickler.load() == "abcd"
-    print(f"reader_index {buf.reader_index} {bytes_io_.tell()}")
-
-    if pa:
-        pa_buf = pa.BufferReader(buf)
-        unpickler = pickle.Unpickler(pa_buf)
-        assert unpickler.load() == b"abc"
-        pa_buf.seek(pa_buf.tell() + 4)
-        assert unpickler.load() == "abcd"
-        print(f"reader_index {buf.reader_index} {pa_buf.tell()} {buf.reader_index}")
-
-    unpickler = pickle.Unpickler(buf)
-    assert unpickler.load() == b"abc"
-    buf.reader_index = buf.reader_index + 4
-    assert unpickler.load() == "abcd"
-    print(f"reader_index {buf.reader_index}")
-
-
 @require_pyarrow
 def test_serialize_arrow():
     record_batch = create_record_batch(10000)
@@ -454,13 +422,16 @@ def test_register_type():
     assert isinstance(fory.deserialize(fory.serialize(A.B.C())), A.B.C)
 
 
-def test_pickle_fallback():
+def test_np_types():
     fory = Fory(language=Language.PYTHON, ref_tracking=True, require_type_registration=False)
     o1 = [1, True, np.dtype(np.int32)]
     data1 = fory.serialize(o1)
     new_o1 = fory.deserialize(data1)
     assert o1 == new_o1
 
+
+def test_pandas_dataframe():
+    fory = Fory(language=Language.PYTHON, ref_tracking=True, require_type_registration=False)
     df = pd.DataFrame({"a": list(range(10))})
     df2 = fory.deserialize(fory.serialize(df))
     assert df2.equals(df)
@@ -469,6 +440,8 @@ def test_pickle_fallback():
 def test_unsupported_callback():
     fory = Fory(language=Language.PYTHON, ref_tracking=True, require_type_registration=False)
 
+    # Test with functions that now have proper serialization support
+    # Functions should no longer be treated as unsupported
     def f1(x):
         return x
 
@@ -478,10 +451,18 @@ def test_unsupported_callback():
     obj1 = [1, True, f1, f2, {1: 2}]
     unsupported_objects = []
     binary1 = fory.serialize(obj1, unsupported_callback=unsupported_objects.append)
-    assert len(unsupported_objects) == 2
-    assert unsupported_objects == [f1, f2]
+    # Functions are now properly supported, so unsupported_objects should be empty
+    assert len(unsupported_objects) == 0
     new_obj1 = fory.deserialize(binary1, unsupported_objects=unsupported_objects)
-    assert new_obj1 == obj1
+    # Functions should roundtrip correctly
+    assert len(new_obj1) == len(obj1)
+    assert new_obj1[0] == obj1[0]  # 1
+    assert new_obj1[1] == obj1[1]  # True
+    assert new_obj1[2](5) == f1(5)  # Test f1 functionality
+    assert new_obj1[3](5) == f2(5)  # Test f2 functionality
+    assert new_obj1[4] == obj1[4]  # {1: 2}
+    # Don't check full equality since functions are new objects after deserialization
+    # The functionality test above already confirmed they work correctly
 
 
 def test_slice():
@@ -535,19 +516,6 @@ def test_duplicate_serialize():
     assert ser_de(fory, EnumClass.E4) == EnumClass.E4
 
 
-@dataclass(unsafe_hash=True)
-class CacheClass1:
-    f1: int
-
-
-def test_cache_serializer():
-    fory = Fory(language=Language.PYTHON, ref_tracking=True)
-    fory.register_type(CacheClass1, serializer=pyfory.PickleStrongCacheSerializer(fory))
-    assert ser_de(fory, CacheClass1(1)) == CacheClass1(1)
-    fory.register_type(CacheClass1, serializer=pyfory.PickleCacheSerializer(fory))
-    assert ser_de(fory, CacheClass1(1)) == CacheClass1(1)
-
-
 def test_pandas_range_index():
     fory = Fory(language=Language.PYTHON, ref_tracking=True, require_type_registration=False)
     fory.register_type(pd.RangeIndex, serializer=pyfory.PandasRangeIndexSerializer(fory))
